package com.nisum.offertransactionservice.client;

import com.nisum.offertransactionservice.config.OfferTransactionConfig;
import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import com.nisum.offertransactionservice.model.PERequest;
import com.nisum.offertransactionservice.model.PEResponse;
import com.safeway.epe.model.Response;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.ClientResponse;
import org.springframework.web.reactive.function.client.ExchangeFilterFunction;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.PostConstruct;

import static com.nisum.offertransactionservice.util.ExceptionUtil.handleError;

@Component
@Slf4j
public class ClientService {


    private WebClient webClient;


    @Autowired
    OfferTransactionConfig offerTransactionConfig;


    @PostConstruct
    public void init() {
        webClient = WebClient.builder().baseUrl(offerTransactionConfig.getPeBaseUrl()).
                defaultHeader(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_VALUE).
                filter(logRequest()).
                filter(logResponse()).
                build();
    }


    private ExchangeFilterFunction logRequest() {
        return ExchangeFilterFunction.ofRequestProcessor(clientRequest -> {
            log.info("Store Consumer Request {}", clientRequest);
            return Mono.just(clientRequest);
        });
    }

    private ExchangeFilterFunction logResponse() {
        return ExchangeFilterFunction.ofResponseProcessor(clientResponse -> {
            log.info("Store Consumer Request {}", clientResponse);
            return Mono.just(clientResponse);
        });
    }



    public Mono<String> getStringMono(String uuid) {
        WebClient build = WebClient.builder().baseUrl(offerTransactionConfig.getScBaseUrl()).build();
        return build.post().
                uri(offerTransactionConfig.getStoreConsumerUrl()).
                accept(MediaType.APPLICATION_JSON).
                body(Mono.just(uuid), String.class).
                retrieve().
                onStatus(HttpStatus::is4xxClientError, clientResponse -> handleError(clientResponse)).
                onStatus(HttpStatus::is5xxServerError, clientResponse -> handleError(clientResponse)).
                bodyToMono(String.class);
    }



    public PEResponse getPeResponseFlux(PERequest peRequest) {
        return webClient.post().
                uri(offerTransactionConfig.getPromotionEngineUrl()).
                accept(MediaType.APPLICATION_JSON).
                contentType(MediaType.APPLICATION_JSON).
                body(Mono.just(peRequest), PERequest.class).
                retrieve().
                onStatus(HttpStatus::is4xxClientError, clientResponse -> handleError(clientResponse)).
                onStatus(HttpStatus::is5xxServerError, clientResponse -> handleError(clientResponse)).
                bodyToFlux(PEResponse.class).
                blockFirst();
    }
}
