package com.nisum.offertransactionservice.client;


import com.nisum.offertransactionservice.model.Item;
import com.nisum.offertransactionservice.model.PERequest;
import com.nisum.offertransactionservice.model.PEResponse;
import com.safeway.epe.model.Employee;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.http.MediaType;
import org.springframework.test.util.ReflectionTestUtils;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ClientServiceTest {


    @Mock
    WebTestClient webTestClient;


    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }


    @Mock
    private WebClient webClientMock;

    @Mock
    private WebClient.RequestHeadersSpec requestHeadersMock;

    @Mock
    private WebClient.RequestHeadersUriSpec requestHeadersUriMock;

    @Mock
    private WebClient.RequestBodySpec requestBodyMock;

    @Mock
    private WebClient.RequestBodyUriSpec requestBodyUriMock;

    @Mock
    private WebClient.ResponseSpec responseMock;



    @InjectMocks
    ClientService clientService;


    @Test
    public void getStringMonoTest(){
        String uuid = "86fd4146-0540-405b-b621-a95f4ccdfa0d";
        ReflectionTestUtils.setField(clientService,"storeConsumerUrl","/store/producer");
        when(webClientMock.post()).thenReturn(requestBodyUriMock);
        when(requestBodyUriMock.uri("/store/producer")).thenReturn(requestBodyMock);
        when(requestBodyMock.bodyValue(Mono.just(uuid))).thenReturn(requestHeadersMock);
        when(requestHeadersMock.retrieve()).thenReturn(responseMock);
        when(responseMock.bodyToMono(String.class)).thenReturn(Mono.just(uuid));
        Mono<String> stringMono = clientService.getStringMono(uuid);
        Assert.assertNotNull(stringMono);
    }


    private PERequest getPEReq() {
        PERequest peRequest = new PERequest();
        peRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05, "cmsCoupons");
        Item item2 = new Item("Thumpsup", "11", 14.05, "cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peRequest.setItemList(itemList);
        return peRequest;
    }



    private PEResponse getPeResponse() {
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05, "cmsCoupons");
        Item item2 = new Item("Thumpsup", "11", 12.32, "cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peResponse.setDiscountedItemList(itemList);
        return peResponse;
    }








}
